@@ -53,7 +53,7 @@ import org.codehaus.plexus.util.xml.Xpp3Dom;
  *
  * @author <a href="snicoll@apache.org">Stephane Nicoll</a>
  * @author <a href="mailto:vincent.siveton@gmail.com">Vincent Siveton</a>
- * @version $Id: PluginReport.java 746372 2009-02-20 21:42:27Z jdcasey $
+ * @version $Id: PluginReport.java 1133737 2011-06-09 09:20:50Z stephenc $
  * @since 2.0
  * @goal report
  * @execute phase="compile"
@@ -91,6 +91,16 @@ public class PluginReport
      */
     protected MojoScanner mojoScanner;
 
+   /**
+    * The file encoding of the source files.
+    *
+    * @parameter expression="${encoding}" default-value="${project.build.sourceEncoding}"
+    *
+    * @since 2.7
+    */
+    private String encoding;
+
+
     /**
      * Specify some requirements to execute this plugin.
      * Example:
@@ -121,6 +131,22 @@ public class PluginReport
      */
     protected String goalPrefix;
 
+    /**
+     * Set this to "true" to skip invoking any goals or reports of the plugin.
+     *
+     * @parameter default-value="false" expression="${maven.plugin.skip}"
+     * @since 2.8
+     */
+    private boolean skip;
+
+    /**
+     * Set this to "true" to skip generating the report.
+     *
+     * @parameter default-value="false" expression="${maven.plugin.report.skip}"
+     * @since 2.8
+     */
+    private boolean skipReport;
+
     /** {@inheritDoc} */
     protected Renderer getSiteRenderer()
     {
@@ -153,6 +179,11 @@ public class PluginReport
         {
             return;
         }
+        if (skip || skipReport)
+        {
+            getLog().info( "Maven Plugin Plugin Report generation skipped." );
+            return;
+        }
 
         // Copy from AbstractGeneratorMojo#execute()
         String defaultGoalPrefix = PluginDescriptor.getGoalPrefixFromArtifactId( project.getArtifactId() );
@@ -183,6 +214,7 @@ public class PluginReport
             pluginDescriptor.setDependencies( PluginUtils.toComponentDependencies( project.getRuntimeDependencies() ) );
             
             PluginToolsRequest request = new DefaultPluginToolsRequest( project, pluginDescriptor );
+            request.setEncoding( encoding );
 
             mojoScanner.populatePluginDescriptor( request );
 
@@ -314,9 +346,10 @@ public class PluginReport
 
 
             boolean hasMavenReport = false;
-            for ( Iterator i = pluginDescriptor.getMojos().iterator(); i.hasNext(); )
+            for ( @SuppressWarnings( "unchecked" )
+            Iterator<MojoDescriptor> i = pluginDescriptor.getMojos().iterator(); i.hasNext(); )
             {
-                MojoDescriptor mojo = (MojoDescriptor) i.next();
+                MojoDescriptor mojo = i.next();
 
                 if ( PluginUtils.isMavenReport( mojo.getImplementation(), project ) )
                 {
@@ -338,13 +371,11 @@ public class PluginReport
                 tableHeader( new String[] { goalColumnName, descriptionColumnName } );
             }
 
-            List mojos = new ArrayList();
+            List<MojoDescriptor> mojos = new ArrayList<MojoDescriptor>();
             mojos.addAll( pluginDescriptor.getMojos() );
             PluginUtils.sortMojos( mojos );
-            for ( Iterator i = mojos.iterator(); i.hasNext(); )
+            for ( MojoDescriptor mojo : mojos )
             {
-                MojoDescriptor mojo = (MojoDescriptor) i.next();
-
                 String goalName = mojo.getFullGoalName();
 
                 /*
