@@ -24,6 +24,7 @@ import org.apache.maven.plugin.descriptor.Parameter;
 import org.apache.maven.plugin.descriptor.PluginDescriptor;
 import org.apache.maven.plugin.descriptor.Requirement;
 import org.apache.maven.tools.plugin.DefaultPluginToolsRequest;
+import org.apache.maven.tools.plugin.ExtendedMojoDescriptor;
 import org.apache.maven.tools.plugin.PluginToolsRequest;
 import org.apache.maven.tools.plugin.util.PluginUtils;
 import org.codehaus.plexus.util.IOUtil;
@@ -36,9 +37,9 @@ import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.OutputStreamWriter;
 import java.io.Writer;
-import java.util.HashMap;
-import java.util.HashSet;
 import java.util.Iterator;
+import java.util.LinkedHashMap;
+import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -48,7 +49,7 @@ import java.util.Set;
  * @todo need to add validation directives so that systems embedding maven2 can
  * get validation directives to help users in IDEs.
  *
- * @version $Id: PluginDescriptorGenerator.java 746372 2009-02-20 21:42:27Z jdcasey $
+ * @version $Id: PluginDescriptorGenerator.java 1133707 2011-06-09 08:28:59Z stephenc $
  */
 public class PluginDescriptorGenerator
     implements Generator
@@ -104,9 +105,10 @@ public class PluginDescriptorGenerator
 
             if ( pluginDescriptor.getMojos() != null )
             {
-                for ( Iterator it = pluginDescriptor.getMojos().iterator(); it.hasNext(); )
+                for ( @SuppressWarnings( "unchecked" )
+                Iterator<MojoDescriptor> it = pluginDescriptor.getMojos().iterator(); it.hasNext(); )
                 {
-                    MojoDescriptor descriptor = (MojoDescriptor) it.next();
+                    MojoDescriptor descriptor = it.next();
                     processMojoDescriptor( descriptor, w );
                 }
             }
@@ -301,24 +303,39 @@ public class PluginDescriptorGenerator
             w.endElement();
         }
 
+        // ----------------------------------------------------------------------
+        // Extended (3.0) descriptor
+        // ----------------------------------------------------------------------
+
+        if ( mojoDescriptor instanceof ExtendedMojoDescriptor )
+        {
+            ExtendedMojoDescriptor extendedMojoDescriptor = (ExtendedMojoDescriptor) mojoDescriptor;
+            if ( extendedMojoDescriptor.getDependencyCollectionRequired() != null )
+            {
+                PluginUtils.element( w, "requiresDependencyCollection",
+                                     extendedMojoDescriptor.getDependencyCollectionRequired() );
+            }
+
+            PluginUtils.element( w, "threadSafe", "" + extendedMojoDescriptor.isThreadSafe() );
+        }
+
         // ----------------------------------------------------------------------
         // Parameters
         // ----------------------------------------------------------------------
 
-        List parameters = mojoDescriptor.getParameters();
+        @SuppressWarnings( "unchecked" )
+        List<Parameter> parameters = mojoDescriptor.getParameters();
 
         w.startElement( "parameters" );
 
-        Map requirements = new HashMap();
+        Map<String, Requirement> requirements = new LinkedHashMap<String, Requirement>();
 
-        Set configuration = new HashSet();
+        Set<Parameter> configuration = new LinkedHashSet<Parameter>();
 
         if ( parameters != null )
         {
-            for ( int j = 0; j < parameters.size(); j++ )
+            for ( Parameter parameter : parameters )
             {
-                Parameter parameter = (Parameter) parameters.get( j );
-
                 String expression = parameter.getExpression();
 
                 if ( StringUtils.isNotEmpty( expression ) && expression.startsWith( "${component." ) )
@@ -405,10 +422,8 @@ public class PluginDescriptorGenerator
         {
             w.startElement( "configuration" );
 
-            for ( Iterator i = configuration.iterator(); i.hasNext(); )
+            for ( Parameter parameter : configuration )
             {
-                Parameter parameter = (Parameter) i.next();
-
                 w.startElement( parameter.getName() );
 
                 String type = parameter.getType();
@@ -441,10 +456,10 @@ public class PluginDescriptorGenerator
         {
             w.startElement( "requirements" );
 
-            for ( Iterator i = requirements.keySet().iterator(); i.hasNext(); )
+            for ( Map.Entry<String, Requirement> entry : requirements.entrySet() )
             {
-                String key = (String) i.next();
-                Requirement requirement = (Requirement) requirements.get( key );
+                String key = entry.getKey();
+                Requirement requirement = entry.getValue();
 
                 w.startElement( "requirement" );
 
