@@ -49,7 +49,7 @@ import org.codehaus.plexus.util.StringUtils;
  * Generates an <code>HelpMojo</code> class.
  *
  * @author <a href="mailto:vincent.siveton@gmail.com">Vincent Siveton</a>
- * @version $Id: PluginHelpGenerator.java 746389 2009-02-20 22:26:33Z bentmann $
+ * @version $Id: PluginHelpGenerator.java 1133707 2011-06-09 08:28:59Z stephenc $
  * @since 2.4
  */
 public class PluginHelpGenerator
@@ -65,6 +65,11 @@ public class PluginHelpGenerator
     /** Default goal */
     private static final String HELP_GOAL = "help";
 
+    private String helpPackageName;
+    
+    /** Flag to indicate if the generated help mojo should use Java 5 features */
+    private boolean useJava5;
+
     /**
      * Default constructor
      */
@@ -98,9 +103,10 @@ public class PluginHelpGenerator
         MojoDescriptor helpDescriptor = makeHelpDescriptor( pluginDescriptor );
 
         // Verify that no help goal already exists
-        for ( Iterator it = pluginDescriptor.getMojos().iterator(); it.hasNext(); )
+        for ( @SuppressWarnings( "unchecked" )
+        Iterator<MojoDescriptor> it = pluginDescriptor.getMojos().iterator(); it.hasNext(); )
         {
-            MojoDescriptor descriptor = (MojoDescriptor) it.next();
+            MojoDescriptor descriptor = it.next();
 
             if ( descriptor.getGoal().equals( helpDescriptor.getGoal() )
                 && !descriptor.getImplementation().equals( helpDescriptor.getImplementation() ) )
@@ -125,7 +131,7 @@ public class PluginHelpGenerator
         try
         {
             writer = new OutputStreamWriter( new FileOutputStream( helpClass ), request.getEncoding() );
-            writeClass( writer, pluginDescriptor, helpDescriptor );
+            writeClass( writer, pluginDescriptor, helpDescriptor, useJava5 );
             writer.flush();
         }
         finally
@@ -134,6 +140,18 @@ public class PluginHelpGenerator
         }
     }
 
+    public PluginHelpGenerator setHelpPackageName( String helpPackageName )
+    {
+        this.helpPackageName = helpPackageName;
+        return this;
+    }
+
+    public PluginHelpGenerator setUseJava5( boolean useJava5 )
+    {
+        this.useJava5 = useJava5;
+        return this;
+    }
+
     // ----------------------------------------------------------------------
     // Private methods
     // ----------------------------------------------------------------------
@@ -145,7 +163,7 @@ public class PluginHelpGenerator
      *            <code>null</code>.
      * @return The mojo descriptor for the generated help goal, never <code>null</code>.
      */
-    private static MojoDescriptor makeHelpDescriptor( PluginDescriptor pluginDescriptor )
+    private MojoDescriptor makeHelpDescriptor( PluginDescriptor pluginDescriptor )
     {
         MojoDescriptor descriptor = new MojoDescriptor();
 
@@ -155,7 +173,11 @@ public class PluginHelpGenerator
 
         descriptor.setGoal( HELP_GOAL );
 
-        String packageName = discoverPackageName( pluginDescriptor );
+        String packageName = helpPackageName;
+        if ( StringUtils.isEmpty( packageName ) )
+        {
+            packageName = discoverPackageName( pluginDescriptor );
+        }
         if ( StringUtils.isNotEmpty( packageName ) )
         {
             descriptor.setImplementation( packageName + '.' + HELP_MOJO_CLASS_NAME );
@@ -219,10 +241,11 @@ public class PluginHelpGenerator
      */
     private static String discoverPackageName( PluginDescriptor pluginDescriptor )
     {
-        Map packageNames = new HashMap();
-        for ( Iterator it = pluginDescriptor.getMojos().iterator(); it.hasNext(); )
+        Map<String, Integer> packageNames = new HashMap<String, Integer>();
+        for ( @SuppressWarnings( "unchecked" )
+        Iterator<MojoDescriptor> it = pluginDescriptor.getMojos().iterator(); it.hasNext(); )
         {
-            MojoDescriptor descriptor = (MojoDescriptor) it.next();
+            MojoDescriptor descriptor = it.next();
 
             String impl = descriptor.getImplementation();
             if ( impl.lastIndexOf( '.' ) != -1 )
@@ -246,9 +269,8 @@ public class PluginHelpGenerator
 
         String packageName = "";
         int max = 0;
-        for ( Iterator it = packageNames.keySet().iterator(); it.hasNext(); )
+        for ( String key : packageNames.keySet() )
         {
-            String key = it.next().toString();
             int value = ( (Integer) packageNames.get( key ) ).intValue();
             if ( value > max )
             {
@@ -266,9 +288,11 @@ public class PluginHelpGenerator
      * @param writer not null
      * @param pluginDescriptor not null
      * @param helpDescriptor not null
+     * @param useJava5 If the generated code should use Java5 features
      * @throws IOException if any
      */
-    private static void writeClass( Writer writer, PluginDescriptor pluginDescriptor, MojoDescriptor helpDescriptor )
+    private static void writeClass( Writer writer, PluginDescriptor pluginDescriptor, MojoDescriptor helpDescriptor,
+                                    boolean useJava5 )
         throws IOException
     {
         String packageName = "";
@@ -291,6 +315,11 @@ public class PluginHelpGenerator
 
         writeMojoJavadoc( writer, pluginDescriptor, helpDescriptor );
 
+        if ( useJava5 )
+        {
+            writer.write( "@SuppressWarnings( \"all\" )" + LS );
+        }
+
         writer.write( "public class " + simpleName + LS );
         writer.write( "    extends AbstractMojo" + LS );
         writer.write( "{" + LS );
@@ -302,7 +331,7 @@ public class PluginHelpGenerator
         writeExecute( writer, pluginDescriptor, helpDescriptor );
 
         writer.write( LS );
-        writeUtilities( writer );
+        writeUtilities( writer, useJava5 );
         writer.write( "}" + LS );
     }
 
@@ -359,6 +388,7 @@ public class PluginHelpGenerator
         writer.write( " * @author " + author.toString() + LS );
         writer.write( " * @goal " + helpDescriptor.getGoal() + LS );
         writer.write( " * @requiresProject false" + LS );
+        writer.write( " * @threadSafe" + LS );
         writer.write( " */" + LS );
     }
 
@@ -370,9 +400,10 @@ public class PluginHelpGenerator
     private static void writeVariables( Writer writer, MojoDescriptor helpDescriptor )
         throws IOException
     {
-        for ( Iterator it = helpDescriptor.getParameters().iterator(); it.hasNext(); )
+        for ( @SuppressWarnings( "unchecked" )
+        Iterator<Parameter> it = helpDescriptor.getParameters().iterator(); it.hasNext(); )
         {
-            Parameter param = (Parameter) it.next();
+            Parameter param = it.next();
             writer.write( "    /**" + LS );
             writer.write( "     * " + StringUtils.escape( param.getDescription() ) + LS );
             writer.write( "     * " + LS );
@@ -405,12 +436,13 @@ public class PluginHelpGenerator
     private static void writeExecute( Writer writer, PluginDescriptor pluginDescriptor, MojoDescriptor helpDescriptor )
         throws IOException
     {
-        List mojoDescriptors = new ArrayList();
+        List<MojoDescriptor> mojoDescriptors = new ArrayList<MojoDescriptor>();
 
         mojoDescriptors.add( helpDescriptor );
-        for ( Iterator it = pluginDescriptor.getMojos().iterator(); it.hasNext(); )
+        for ( @SuppressWarnings( "unchecked" )
+        Iterator<MojoDescriptor> it = pluginDescriptor.getMojos().iterator(); it.hasNext(); )
         {
-            MojoDescriptor mojoDescriptor = (MojoDescriptor) it.next();
+            MojoDescriptor mojoDescriptor = it.next();
 
             if ( !helpDescriptor.getGoal().equals( mojoDescriptor.getGoal() ) )
             {
@@ -480,9 +512,9 @@ public class PluginHelpGenerator
 
         writer.write( LS );
 
-        for ( Iterator it = mojoDescriptors.iterator(); it.hasNext(); )
+        for ( Iterator<MojoDescriptor> it = mojoDescriptors.iterator(); it.hasNext(); )
         {
-            MojoDescriptor descriptor = (MojoDescriptor) it.next();
+            MojoDescriptor descriptor = it.next();
 
             writeGoal( writer, descriptor );
         }
@@ -527,7 +559,8 @@ public class PluginHelpGenerator
 
         if ( descriptor.getParameters() != null && descriptor.getParameters().size() > 0 )
         {
-            List params = descriptor.getParameters();
+            @SuppressWarnings( "unchecked" )
+            List<Parameter> params = descriptor.getParameters();
 
             PluginUtils.sortMojoParameters( params );
 
@@ -537,10 +570,8 @@ public class PluginHelpGenerator
             writer.write( "                append( sb, \"Available parameters:\", 1 );" + LS );
             writer.write( "                append( sb, \"\", 0 );" + LS );
 
-            for ( Iterator it = params.iterator(); it.hasNext(); )
+            for ( Parameter parameter : params )
             {
-                Parameter parameter = (Parameter) it.next();
-
                 if ( parameter.isEditable() )
                 {
                     writer.write( LS );
@@ -569,11 +600,12 @@ public class PluginHelpGenerator
         {
             String parameterName = StringUtils.escape( parameter.getName() );
             String parameterDescription = toDescription( parameter.getDescription() );
-            String parameterDefaultValue = parameterName
-                + ( StringUtils.isNotEmpty( parameter.getDefaultValue() ) ? " (Default: "
-                    + StringUtils.escape( parameter.getDefaultValue() ) + ")" : "" );
-
-            writer.write( "                append( sb, \"" + parameterDefaultValue + "\", 2 );" + LS );
+            String parameterDefaultValue = "";
+            if ( StringUtils.isNotEmpty( parameter.getDefaultValue() ) )
+            {
+                parameterDefaultValue = " (Default: " + StringUtils.escape( parameter.getDefaultValue() ) + ")";
+            }
+            writer.write( "                append( sb, \"" + parameterName + parameterDefaultValue + "\", 2 );" + LS );
             if ( StringUtils.isNotEmpty( parameter.getDeprecated() ) )
             {
                 writer.write( "                append( sb, \"Deprecated. " + toDescription( parameter.getDeprecated() )
@@ -581,15 +613,25 @@ public class PluginHelpGenerator
                 writer.write( "                append( sb, \"\", 0 );" + LS );
             }
             writer.write( "                append( sb, \"" + parameterDescription + "\", 3 );" + LS );
+            if ( parameter.isRequired() )
+            {
+                writer.write( "                append( sb, \"Required: Yes\", 3 );" + LS );
+            }
+            if ( StringUtils.isNotEmpty( parameter.getExpression() ) )
+            {
+                writer.write( "                append( sb, \"Expression: "
+                    + StringUtils.escape( parameter.getExpression() ) + "\", 3 );" + LS );
+            }
             writer.write( "                append( sb, \"\", 0 );" + LS );
         }
     }
 
     /**
      * @param writer not null
+     * @param useJava5 If the generated code should use Java5 features
      * @throws IOException if any
      */
-    private static void writeUtilities( Writer writer )
+    private static void writeUtilities( Writer writer, boolean useJava5 )
         throws IOException
     {
         writer.write( "    /**" + LS );
@@ -646,7 +688,14 @@ public class PluginHelpGenerator
         writer.write( "    private static List toLines( String text, int indent, int indentSize, int lineLength )"
             + LS );
         writer.write( "    {" + LS );
-        writer.write( "        List lines = new ArrayList();" + LS );
+        if ( useJava5 )
+        {
+            writer.write( "        List<String> lines = new ArrayList<String>();" + LS );
+        }
+        else
+        {
+            writer.write( "        List lines = new ArrayList();" + LS );
+        }
         writer.write( LS );
         writer.write( "        String ind = repeat( \"\\t\", indent );" + LS );
         writer.write( "        String[] plainLines = text.split( \"(\\r\\n)|(\\r)|(\\n)\" );" + LS );
@@ -668,8 +717,16 @@ public class PluginHelpGenerator
         writer.write( "     * @param indentSize The size of each indentation, must not be negative." + LS );
         writer.write( "     * @param lineLength The length of the line, must not be negative." + LS );
         writer.write( "     */" + LS );
-        writer.write( "    private static void toLines( List lines, String line, int indentSize, int lineLength )"
-            + LS );
+        if ( useJava5 )
+        {
+            writer.write( "    private static void toLines( List<String> lines, String line, int indentSize, int lineLength )"
+                + LS );
+        }
+        else
+        {
+            writer.write( "    private static void toLines( List lines, String line, int indentSize, int lineLength )"
+                + LS );
+        }
         writer.write( "    {" + LS );
         writer.write( "        int lineIndent = getIndentLevel( line );" + LS );
         writer.write( "        StringBuffer buf = new StringBuffer( 256 );" + LS );
