@@ -37,6 +37,7 @@ import org.apache.maven.plugin.descriptor.Parameter;
 import org.apache.maven.plugin.descriptor.PluginDescriptor;
 import org.apache.maven.project.MavenProject;
 import org.apache.maven.tools.plugin.DefaultPluginToolsRequest;
+import org.apache.maven.tools.plugin.ExtendedMojoDescriptor;
 import org.apache.maven.tools.plugin.PluginToolsRequest;
 import org.apache.maven.tools.plugin.util.PluginUtils;
 import org.codehaus.plexus.util.IOUtil;
@@ -46,7 +47,7 @@ import org.codehaus.plexus.util.xml.XMLWriter;
 
 /**
  * @todo add example usage tag that can be shown in the doco
- * @version $Id: PluginXdocGenerator.java 746372 2009-02-20 21:42:27Z jdcasey $
+ * @version $Id: PluginXdocGenerator.java 1133707 2011-06-09 08:28:59Z stephenc $
  */
 public class PluginXdocGenerator
     implements Generator
@@ -101,16 +102,17 @@ public class PluginXdocGenerator
     {
         execute( destinationDirectory, new DefaultPluginToolsRequest( project, pluginDescriptor ) );
     }
-    
+
     /** {@inheritDoc} */
     public void execute( File destinationDirectory, PluginToolsRequest request )
         throws IOException
     {
         if ( request.getPluginDescriptor().getMojos() != null )
         {
-            for ( Iterator it = request.getPluginDescriptor().getMojos().iterator(); it.hasNext(); )
+            for ( @SuppressWarnings( "unchecked" )
+            Iterator<MojoDescriptor> it = request.getPluginDescriptor().getMojos().iterator(); it.hasNext(); )
             {
-                MojoDescriptor descriptor = (MojoDescriptor) it.next();
+                MojoDescriptor descriptor = it.next();
 
                 processMojoDescriptor( descriptor, destinationDirectory );
             }
@@ -160,6 +162,10 @@ public class PluginXdocGenerator
     private void writeBody( MojoDescriptor mojoDescriptor, XMLWriter w )
     {
         w.startElement( "document" );
+        w.addAttribute( "xmlns", "http://maven.apache.org/XDOC/2.0" );
+        w.addAttribute( "xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance" );
+        w.addAttribute( "xsi:schemaLocation",
+                        "http://maven.apache.org/XDOC/2.0 http://maven.apache.org/xsd/xdoc-2.0.xsd" );
 
         // ----------------------------------------------------------------------
         //
@@ -254,12 +260,15 @@ public class PluginXdocGenerator
         w.writeMarkup( getString( "pluginxdoc.mojodescriptor.attributes" ) );
         w.endElement(); //p
 
-        w.startElement( "ul" );
-
+        boolean addedUl = false;
         String value;
-
         if ( mojoDescriptor.isProjectRequired() )
         {
+            if ( !addedUl )
+            {
+                w.startElement( "ul" );
+                addedUl = true;
+            }
             w.startElement( "li" );
             w.writeMarkup( getString( "pluginxdoc.mojodescriptor.projectRequired" ) );
             w.endElement(); //li
@@ -267,6 +276,11 @@ public class PluginXdocGenerator
 
         if ( mojoDescriptor.isAggregator() )
         {
+            if ( !addedUl )
+            {
+                w.startElement( "ul" );
+                addedUl = true;
+            }
             w.startElement( "li" );
             w.writeMarkup( getString( "pluginxdoc.mojodescriptor.aggregator" ) );
             w.endElement(); //li
@@ -274,6 +288,11 @@ public class PluginXdocGenerator
 
         if ( mojoDescriptor.isDirectInvocationOnly() )
         {
+            if ( !addedUl )
+            {
+                w.startElement( "ul" );
+                addedUl = true;
+            }
             w.startElement( "li" );
             w.writeMarkup( getString( "pluginxdoc.mojodescriptor.directInvocationOnly" ) );
             w.endElement(); //li
@@ -282,14 +301,55 @@ public class PluginXdocGenerator
         value = mojoDescriptor.isDependencyResolutionRequired();
         if ( StringUtils.isNotEmpty( value ) )
         {
+            if ( !addedUl )
+            {
+                w.startElement( "ul" );
+                addedUl = true;
+            }
             w.startElement( "li" );
             w.writeMarkup( format( "pluginxdoc.mojodescriptor.dependencyResolutionRequired", value ) );
             w.endElement(); //li
         }
 
+        if ( mojoDescriptor instanceof ExtendedMojoDescriptor )
+        {
+            ExtendedMojoDescriptor extendedMojoDescriptor = (ExtendedMojoDescriptor) mojoDescriptor;
+
+            value = extendedMojoDescriptor.getDependencyCollectionRequired();
+            if ( StringUtils.isNotEmpty( value ) )
+            {
+                if ( !addedUl )
+                {
+                    w.startElement( "ul" );
+                    addedUl = true;
+                }
+                w.startElement( "li" );
+                w.writeMarkup( format( "pluginxdoc.mojodescriptor.dependencyCollectionRequired", value ) );
+                w.endElement(); //li
+            }
+
+            if ( extendedMojoDescriptor.isThreadSafe() )
+            {
+                if ( !addedUl )
+                {
+                    w.startElement( "ul" );
+                    addedUl = true;
+                }
+                w.startElement( "li" );
+                w.writeMarkup( getString( "pluginxdoc.mojodescriptor.threadSafe" ) );
+                w.endElement(); //li
+            }
+
+        }
+
         value = mojoDescriptor.getSince();
         if ( StringUtils.isNotEmpty( value ) )
         {
+            if ( !addedUl )
+            {
+                w.startElement( "ul" );
+                addedUl = true;
+            }
             w.startElement( "li" );
             w.writeMarkup( format( "pluginxdoc.mojodescriptor.since", value ) );
             w.endElement(); //li
@@ -298,6 +358,11 @@ public class PluginXdocGenerator
         value = mojoDescriptor.getPhase();
         if ( StringUtils.isNotEmpty( value ) )
         {
+            if ( !addedUl )
+            {
+                w.startElement( "ul" );
+                addedUl = true;
+            }
             w.startElement( "li" );
             w.writeMarkup( format( "pluginxdoc.mojodescriptor.phase", value ) );
             w.endElement(); //li
@@ -306,6 +371,11 @@ public class PluginXdocGenerator
         value = mojoDescriptor.getExecutePhase();
         if ( StringUtils.isNotEmpty( value ) )
         {
+            if ( !addedUl )
+            {
+                w.startElement( "ul" );
+                addedUl = true;
+            }
             w.startElement( "li" );
             w.writeMarkup( format( "pluginxdoc.mojodescriptor.executePhase", value ) );
             w.endElement(); //li
@@ -314,6 +384,11 @@ public class PluginXdocGenerator
         value = mojoDescriptor.getExecuteGoal();
         if ( StringUtils.isNotEmpty( value ) )
         {
+            if ( !addedUl )
+            {
+                w.startElement( "ul" );
+                addedUl = true;
+            }
             w.startElement( "li" );
             w.writeMarkup( format( "pluginxdoc.mojodescriptor.executeGoal", value ) );
             w.endElement(); //li
@@ -322,6 +397,11 @@ public class PluginXdocGenerator
         value = mojoDescriptor.getExecuteLifecycle();
         if ( StringUtils.isNotEmpty( value ) )
         {
+            if ( !addedUl )
+            {
+                w.startElement( "ul" );
+                addedUl = true;
+            }
             w.startElement( "li" );
             w.writeMarkup( format( "pluginxdoc.mojodescriptor.executeLifecycle", value ) );
             w.endElement(); //li
@@ -329,6 +409,11 @@ public class PluginXdocGenerator
 
         if ( mojoDescriptor.isOnlineRequired() )
         {
+            if ( !addedUl )
+            {
+                w.startElement( "ul" );
+                addedUl = true;
+            }
             w.startElement( "li" );
             w.writeMarkup( getString( "pluginxdoc.mojodescriptor.onlineRequired" ) );
             w.endElement(); //li
@@ -336,12 +421,20 @@ public class PluginXdocGenerator
 
         if ( !mojoDescriptor.isInheritedByDefault() )
         {
+            if ( !addedUl )
+            {
+                w.startElement( "ul" );
+                addedUl = true;
+            }
             w.startElement( "li" );
             w.writeMarkup( getString( "pluginxdoc.mojodescriptor.inheritedByDefault" ) );
             w.endElement(); //li
         }
 
-        w.endElement(); //ul
+        if ( addedUl )
+        {
+            w.endElement(); //ul
+        }
     }
 
     /**
@@ -350,10 +443,11 @@ public class PluginXdocGenerator
      */
     private void writeGoalParameterTable( MojoDescriptor mojoDescriptor, XMLWriter w )
     {
-        List parameterList = mojoDescriptor.getParameters();
+        @SuppressWarnings( "unchecked" )
+        List<Parameter> parameterList = mojoDescriptor.getParameters();
 
         //remove components and read-only parameters
-        List list = filterParameters( parameterList );
+        List<Parameter> list = filterParameters( parameterList );
 
         if ( list != null && list.size() > 0 )
         {
@@ -378,16 +472,14 @@ public class PluginXdocGenerator
      * @param parameterList not null
      * @return the parameters list without components.
      */
-    private List filterParameters( List parameterList )
+    private List<Parameter> filterParameters( List<Parameter> parameterList )
     {
-        List filtered = new ArrayList();
+        List<Parameter> filtered = new ArrayList<Parameter>();
 
         if ( parameterList != null )
         {
-            for ( Iterator parameters = parameterList.iterator(); parameters.hasNext(); )
+            for ( Parameter parameter :  parameterList )
             {
-                Parameter parameter = (Parameter) parameters.next();
-
                 if ( parameter.isEditable() )
                 {
                     String expression = parameter.getExpression();
@@ -408,14 +500,14 @@ public class PluginXdocGenerator
      * @param parameterList not null
      * @param w not null
      */
-    private void writeParameterDetails( MojoDescriptor mojoDescriptor, List parameterList, XMLWriter w )
+    private void writeParameterDetails( MojoDescriptor mojoDescriptor, List<Parameter> parameterList, XMLWriter w )
     {
         w.startElement( "subsection" );
         w.addAttribute( "name", getString( "pluginxdoc.mojodescriptor.parameter.details" ) );
 
-        for ( Iterator parameters = parameterList.iterator(); parameters.hasNext(); )
+        for ( Iterator<Parameter> parameters = parameterList.iterator(); parameters.hasNext(); )
         {
-            Parameter parameter = (Parameter) parameters.next();
+            Parameter parameter = parameters.next();
 
             w.startElement( "p" );
             w.writeMarkup( format( "pluginxdoc.mojodescriptor.parameter.name_internal", parameter.getName() ) );
@@ -440,18 +532,32 @@ public class PluginXdocGenerator
             }
             w.endElement(); // div
 
-            w.startElement( "ul" );
-
+            boolean addedUl = false;
+            if ( !addedUl && StringUtils.isNotEmpty( parameter.getType() ) )
+            {
+                w.startElement( "ul" );
+                addedUl = true;
+            }
             writeDetail( getString( "pluginxdoc.mojodescriptor.parameter.type" ), parameter.getType(), w );
 
             if ( StringUtils.isNotEmpty( parameter.getSince() ) )
             {
+                if ( !addedUl )
+                {
+                    w.startElement( "ul" );
+                    addedUl = true;
+                }
                 writeDetail( getString( "pluginxdoc.mojodescriptor.parameter.since" ), parameter.getSince(), w );
             }
             else
             {
                 if ( StringUtils.isNotEmpty( mojoDescriptor.getSince() ) )
                 {
+                    if ( !addedUl )
+                    {
+                        w.startElement( "ul" );
+                        addedUl = true;
+                    }
                     writeDetail( getString( "pluginxdoc.mojodescriptor.parameter.since" ),
                                  mojoDescriptor.getSince(), w );
                 }
@@ -459,20 +565,44 @@ public class PluginXdocGenerator
 
             if ( parameter.isRequired() )
             {
+                if ( !addedUl )
+                {
+                    w.startElement( "ul" );
+                    addedUl = true;
+                }
                 writeDetail( getString( "pluginxdoc.mojodescriptor.parameter.required" ),
                              getString( "pluginxdoc.yes" ), w );
             }
             else
             {
+                if ( !addedUl )
+                {
+                    w.startElement( "ul" );
+                    addedUl = true;
+                }
                 writeDetail( getString( "pluginxdoc.mojodescriptor.parameter.required" ),
                              getString( "pluginxdoc.no" ), w );
             }
 
+            if ( !addedUl && StringUtils.isNotEmpty( parameter.getExpression() ) )
+            {
+                w.startElement( "ul" );
+                addedUl = true;
+            }
             writeDetail( getString( "pluginxdoc.mojodescriptor.parameter.expression" ), parameter.getExpression(), w );
 
-            writeDetail( getString( "pluginxdoc.mojodescriptor.parameter.default" ), parameter.getDefaultValue(), w );
+            if ( !addedUl && StringUtils.isNotEmpty( parameter.getDefaultValue() ) )
+            {
+                w.startElement( "ul" );
+                addedUl = true;
+            }
+            writeDetail( getString( "pluginxdoc.mojodescriptor.parameter.default" ),
+                         escapeXml( parameter.getDefaultValue() ), w );
 
-            w.endElement(); //ul
+            if ( addedUl )
+            {
+                w.endElement(); //ul
+            }
 
             if ( parameters.hasNext() )
             {
@@ -503,16 +633,16 @@ public class PluginXdocGenerator
      * @param parameterList not null
      * @param w not null
      */
-    private void writeParameterSummary( MojoDescriptor mojoDescriptor, List parameterList, XMLWriter w )
+    private void writeParameterSummary( MojoDescriptor mojoDescriptor, List<Parameter> parameterList, XMLWriter w )
     {
-        List requiredParams = getParametersByRequired( true, parameterList );
+        List<Parameter> requiredParams = getParametersByRequired( true, parameterList );
         if ( requiredParams.size() > 0 )
         {
             writeParameterList( mojoDescriptor, getString( "pluginxdoc.mojodescriptor.requiredParameters" ),
                                 requiredParams, w );
         }
 
-        List optionalParams = getParametersByRequired( false, parameterList );
+        List<Parameter> optionalParams = getParametersByRequired( false, parameterList );
         if ( optionalParams.size() > 0 )
         {
             writeParameterList( mojoDescriptor, getString( "pluginxdoc.mojodescriptor.optionalParameters" ),
@@ -526,14 +656,13 @@ public class PluginXdocGenerator
      * @param parameterList not null
      * @param w not null
      */
-    private void writeParameterList( MojoDescriptor mojoDescriptor, String title, List parameterList, XMLWriter w )
+    private void writeParameterList( MojoDescriptor mojoDescriptor, String title, List<Parameter> parameterList, XMLWriter w )
     {
         w.startElement( "subsection" );
         w.addAttribute( "name", title );
 
         w.startElement( "table" );
         w.addAttribute( "border", "0" );
-        w.addAttribute( "align", "left" );
 
         w.startElement( "tr" );
         w.startElement( "th" );
@@ -550,10 +679,8 @@ public class PluginXdocGenerator
         w.endElement(); //th
         w.endElement(); //tr
 
-        for ( Iterator parameters = parameterList.iterator(); parameters.hasNext(); )
+        for ( Parameter parameter : parameterList )
         {
-            Parameter parameter = (Parameter) parameters.next();
-
             w.startElement( "tr" );
             w.startElement( "td" );
             w.writeMarkup( format( "pluginxdoc.mojodescriptor.parameter.name_link", parameter.getName() ) );
@@ -600,7 +727,7 @@ public class PluginXdocGenerator
             if ( StringUtils.isNotEmpty( parameter.getDefaultValue() ) )
             {
                 w.writeMarkup( format( "pluginxdoc.mojodescriptor.parameter.defaultValue",
-                                       parameter.getDefaultValue() ) );
+                                       escapeXml( parameter.getDefaultValue() ) ) );
             }
             w.endElement(); //td
             w.endElement(); //tr
@@ -615,14 +742,12 @@ public class PluginXdocGenerator
      * @param parameterList not null
      * @return list of parameters depending the value of <code>required</code>
      */
-    private List getParametersByRequired( boolean required, List parameterList )
+    private List<Parameter> getParametersByRequired( boolean required, List<Parameter> parameterList )
     {
-        List list = new ArrayList();
+        List<Parameter> list = new ArrayList<Parameter>();
 
-        for ( Iterator parameters = parameterList.iterator(); parameters.hasNext(); )
+        for ( Parameter parameter : parameterList )
         {
-            Parameter parameter = (Parameter) parameters.next();
-
             if ( parameter.isRequired() == required )
             {
                 list.add( parameter );
@@ -675,10 +800,32 @@ public class PluginXdocGenerator
      */
     private String format( String key, Object[] args )
     {
+        String pattern = getString( key );
+        // we don't need quoting so spare us the confusion in the resource bundle to double them up in some keys
+        pattern = StringUtils.replace( pattern, "'", "''" );
+
         MessageFormat messageFormat = new MessageFormat( "" );
         messageFormat.setLocale( locale );
-        messageFormat.applyPattern( getBundle().getString( key ) );
+        messageFormat.applyPattern( pattern );
 
         return messageFormat.format( args );
     }
+
+    /**
+     * @param text the string to escape
+     * @return A string escaped with XML entities
+     */
+    private String escapeXml( String text )
+    {
+        if ( text != null )
+        {
+            text = text.replaceAll( "&", "&amp;" );
+            text = text.replaceAll( "<", "&lt;" );
+            text = text.replaceAll( ">", "&gt;" );
+            text = text.replaceAll( "\"", "&quot;" );
+            text = text.replaceAll( "\'", "&apos;" );
+        }
+        return text;
+    }
+
 }
