@@ -33,6 +33,7 @@ import org.apache.maven.plugin.descriptor.PluginDescriptor;
 import org.apache.maven.plugin.descriptor.Requirement;
 import org.apache.maven.project.MavenProject;
 import org.apache.maven.tools.plugin.DefaultPluginToolsRequest;
+import org.apache.maven.tools.plugin.ExtendedMojoDescriptor;
 import org.apache.maven.tools.plugin.PluginToolsRequest;
 import org.apache.maven.tools.plugin.extractor.MojoDescriptorExtractor;
 import org.apache.maven.tools.plugin.extractor.ExtractionException;
@@ -56,7 +57,7 @@ import java.util.TreeMap;
  *
  * @todo need to add validation directives so that systems embedding maven2 can
  * get validation directives to help users in IDEs.
- * @version $Id: JavaMojoDescriptorExtractor.java 746400 2009-02-20 22:42:13Z bentmann $
+ * @version $Id: JavaMojoDescriptorExtractor.java 1133707 2011-06-09 08:28:59Z stephenc $
  * @see org.apache.maven.plugin.descriptor.MojoDescriptor
  */
 public class JavaMojoDescriptorExtractor
@@ -198,7 +199,7 @@ public class JavaMojoDescriptorExtractor
     protected MojoDescriptor createMojoDescriptor( JavaClass javaClass )
         throws InvalidPluginDescriptorException
     {
-        MojoDescriptor mojoDescriptor = new MojoDescriptor();
+        ExtendedMojoDescriptor mojoDescriptor = new ExtendedMojoDescriptor();
         mojoDescriptor.setLanguage( "java" );
         mojoDescriptor.setImplementation( javaClass.getFullyQualifiedName() );
         mojoDescriptor.setDescription( javaClass.getComment() );
@@ -303,6 +304,21 @@ public class JavaMojoDescriptorExtractor
             mojoDescriptor.setDependencyResolutionRequired( v );
         }
 
+        // Dependency collection flag
+        DocletTag requiresDependencyCollection =
+            findInClassHierarchy( javaClass, JavaMojoAnnotation.REQUIRES_DEPENDENCY_COLLECTION );
+        if ( requiresDependencyCollection != null )
+        {
+            String v = requiresDependencyCollection.getValue();
+
+            if ( StringUtils.isEmpty( v ) )
+            {
+                v = "runtime";
+            }
+
+            mojoDescriptor.setDependencyCollectionRequired( v );
+        }
+
         // requiresDirectInvocation flag
         value =
             getBooleanTagValue( javaClass, JavaMojoAnnotation.REQUIRES_DIRECT_INVOCATION,
@@ -342,6 +358,11 @@ public class JavaMojoDescriptorExtractor
             mojoDescriptor.setSince( since.getValue() );
         }
 
+        // Thread-safe mojo 
+
+        value = getBooleanTagValue( javaClass, JavaMojoAnnotation.THREAD_SAFE, true, mojoDescriptor.isThreadSafe() );
+        mojoDescriptor.setThreadSafe( value );
+
         extractParameters( mojoDescriptor, javaClass );
 
         return mojoDescriptor;
@@ -370,6 +391,35 @@ public class JavaMojoDescriptorExtractor
         return defaultValue;
     }
 
+    /**
+     * @param javaClass     not null
+     * @param tagName       not null
+     * @param defaultForTag The wanted default value when only the tagname is present
+     * @param defaultValue  the wanted default value when the tag is not specified
+     * @return the boolean value of the given tagName
+     * @see #findInClassHierarchy(JavaClass, String)
+     */
+    private static boolean getBooleanTagValue( JavaClass javaClass, String tagName, boolean defaultForTag,
+                                               boolean defaultValue )
+    {
+        DocletTag tag = findInClassHierarchy( javaClass, tagName );
+
+        if ( tag != null )
+        {
+            String value = tag.getValue();
+
+            if ( StringUtils.isNotEmpty( value ) )
+            {
+                return Boolean.valueOf( value ).booleanValue();
+            }
+            else
+            {
+                return defaultForTag;
+            }
+        }
+        return defaultValue;
+    }
+
     /**
      * @param javaClass not null
      * @param tagName not null
@@ -404,13 +454,11 @@ public class JavaMojoDescriptorExtractor
         // We're resolving class-level, ancestor-class-field, local-class-field order here.
         // ---------------------------------------------------------------------------------
 
-        Map rawParams = extractFieldParameterTags( javaClass );
+        Map<String, JavaField> rawParams = extractFieldParameterTags( javaClass );
 
-        for ( Iterator it = rawParams.entrySet().iterator(); it.hasNext(); )
+        for ( Map.Entry<String, JavaField> entry : rawParams.entrySet() )
         {
-            Map.Entry entry = (Map.Entry) it.next();
-
-            JavaField field = (JavaField) entry.getValue();
+            JavaField field = entry.getValue();
 
             Type type = field.getType();
 
@@ -456,7 +504,7 @@ public class JavaMojoDescriptorExtractor
 
                 pd.setRequirement( new Requirement( role, roleHint ) );
 
-                pd.setName( (String) entry.getKey() );
+                pd.setName( entry.getKey() );
 
                 pd.setEditable( false );
                 /* TODO: or better like this? Need @component fields be editable for the user?
@@ -485,7 +533,7 @@ public class JavaMojoDescriptorExtractor
                 }
                 else
                 {
-                    pd.setName( (String) entry.getKey() );
+                    pd.setName( entry.getKey() );
                 }
 
                 pd.setRequired( field.getTagByName( JavaMojoAnnotation.REQUIRED ) != null );
@@ -543,9 +591,9 @@ public class JavaMojoDescriptorExtractor
      * @param javaClass not null
      * @return map with Mojo parameters names as keys
      */
-    private Map extractFieldParameterTags( JavaClass javaClass )
+    private Map<String, JavaField> extractFieldParameterTags( JavaClass javaClass )
     {
-        Map rawParams;
+        Map<String, JavaField> rawParams;
 
         // we have to add the parent fields first, so that they will be overwritten by the local fields if
         // that actually happens...
@@ -557,7 +605,7 @@ public class JavaMojoDescriptorExtractor
         }
         else
         {
-            rawParams = new TreeMap();
+            rawParams = new TreeMap<String, JavaField>();
         }
 
         JavaField[] classFields = javaClass.getFields();
@@ -579,19 +627,19 @@ public class JavaMojoDescriptorExtractor
     }
 
     /** {@inheritDoc} */
-    public List execute( MavenProject project, PluginDescriptor pluginDescriptor )
+    public List<MojoDescriptor> execute( MavenProject project, PluginDescriptor pluginDescriptor )
         throws ExtractionException, InvalidPluginDescriptorException
     {
         return execute( new DefaultPluginToolsRequest( project, pluginDescriptor ) );
     }
     
     /** {@inheritDoc} */
-    public List execute( PluginToolsRequest request )
+    public List<MojoDescriptor> execute( PluginToolsRequest request )
         throws ExtractionException, InvalidPluginDescriptorException
     {
         JavaClass[] javaClasses = discoverClasses( request );
 
-        List descriptors = new ArrayList();
+        List<MojoDescriptor> descriptors = new ArrayList<MojoDescriptor>();
 
         for ( int i = 0; i < javaClasses.length; i++ )
         {
@@ -623,9 +671,10 @@ public class JavaMojoDescriptorExtractor
         
         MavenProject project = request.getProject();
 
-        for ( Iterator i = project.getCompileSourceRoots().iterator(); i.hasNext(); )
+        for ( @SuppressWarnings( "unchecked" )
+        Iterator<String> i = project.getCompileSourceRoots().iterator(); i.hasNext(); )
         {
-            builder.addSourceTree( new File( (String) i.next() ) );
+            builder.addSourceTree( new File( i.next() ) );
         }
 
         // TODO be more dynamic
@@ -645,13 +694,14 @@ public class JavaMojoDescriptorExtractor
     protected void validate( MojoDescriptor mojoDescriptor )
         throws InvalidParameterException
     {
-        List parameters = mojoDescriptor.getParameters();
+        @SuppressWarnings( "unchecked" )
+        List<Parameter> parameters = mojoDescriptor.getParameters();
 
         if ( parameters != null )
         {
             for ( int j = 0; j < parameters.size(); j++ )
             {
-                validateParameter( (Parameter) parameters.get( j ), j );
+                validateParameter( parameters.get( j ), j );
             }
         }
     }
